@@ -50,8 +50,6 @@
 #define VOID            (TOKEN_C_START + 23)
 #define VOLATILE        (TOKEN_C_START + 24)
 
-#define CHAR_CONST      (TOKEN_C_START + 25)
-#define STRING_CONST    (TOKEN_C_START + 26)
 
 #define DEC_OP          (TOKEN_C_START + 27)
 #define INC_OP          (TOKEN_C_START + 28)
