@@ -41,7 +41,7 @@ static accumulator_t *      accumulators = (accumulator_t *)NULL;
 static void analyze_variable(void);
 static void analyze_identifier(void);
 static void analyze_comment(void);
-static void analyze_python_backticks(void);
+static void analyze_backticks(void);
 static argument_t *get_argument(int number);
 
 static
@@ -187,10 +187,7 @@ void scan_tokens(processorfn_t fn, void *arg)
                 break;
 
             case '`':
-                if (ratslexer.lang == LANG_PYTHON)  
-                {
-                  analyze_python_backticks();
-                }
+                analyze_backticks();
                 break;
  
         }
@@ -207,16 +204,34 @@ int gobble_backtick(int token, void *arg)
 }
 
 static void
-analyze_python_backticks(void)
+analyze_backticks(void)
 {
   int myline = *ratslexer.lex_lineno;
+
+  if (ratslexer.lang == LANG_C)  
+  {
+    return;
+  }
+  
   /* we're just gobbling up whatever is in the backticks and
      ignoring them for now.
   */
 
   scan_tokens(gobble_backtick, NULL);
 
-  log_pythonbacktick(myline, Medium);
+  switch(ratslexer.lang)  {
+
+    case LANG_PYTHON:
+                        log_pythonbacktick(myline, Medium);
+                        break;
+    case LANG_PHP:    
+                        log_phpbacktick(myline, Medium);
+                        break;
+    case LANG_PERL:
+                        log_perlbacktick(myline, Medium);
+                        break;
+  }
+
 }
 
 
@@ -429,7 +444,7 @@ int check_argument(int token, void *arg)
 
     if (data->current->is_constant)
     {
-        if (token < HEX_CONST || token > STRING_CONST)
+        if (token < CONST_START || token > CONST_END)
             data->current->is_constant = 0;
         else if (token == STRING_CONST && !data->current->contains_ps)
             data->current->contains_ps = check_format_string(*ratslexer.yytext, -1);
@@ -471,7 +486,6 @@ Vuln_t *locate_identifier(char *identifier)
 
     Vuln_t *data = NULL;
     
-     
     if (ratslexer.langhash) 
         data = HashGet(ratslexer.langhash, identifier);
     if ((data == NULL) && defaultdb)
@@ -614,9 +628,12 @@ void analyze_identifier(void)
     if (data->InputProblem != (InputProblem_t *)NULL)
     {
         analyzed++;
-        if ((arg = get_argument(data->InputProblem->Arg)) != (argument_t *)NULL)
-            if (!arg->is_constant)
+        if ((arg = get_argument(data->InputProblem->Arg)) != (argument_t *)NULL){
+            if (!arg->is_constant)  {
+ 
                 log_vulnerability(InputProblem, data->InputProblem->Severity);
+            } 
+        }
     }
 
     if (data->RaceCheck > 0)
@@ -745,6 +762,19 @@ void setup_c(FILE *fd)
 }
 
 
+void setup_php(FILE *fd)
+{
+
+    yyphpin = fd;
+    ratslexer.yytext = &yyphptext;
+    ratslexer.yyin = fd;
+    ratslexer.yylex = yyphplex;
+    ratslexer.yycomment = &yyphpcomment;
+    ratslexer.lex_lineno = &phplex_lineno;
+    ratslexer.langhash = (Hash)HashGet(database, "php");
+    ratslexer.lang = LANG_PHP;
+}
+
 
 void determine_language(char *filename, FILE *fd, int forcelang)
 {
@@ -759,7 +789,7 @@ void determine_language(char *filename, FILE *fd, int forcelang)
           setup_python(fd);
       else if (forcelang == LANG_C)
           setup_c(fd);
-      else if (forcelang = LANG_PERL)
+      else if (forcelang == LANG_PERL)
           setup_perl(fd);
       return;
   }
@@ -772,6 +802,8 @@ void determine_language(char *filename, FILE *fd, int forcelang)
     setup_python(fd);
   } else if (!strcmp(dot, ".pl") || !strcmp(dot, ".pm"))  {
     setup_perl(fd);
+  } else if (!strcmp(dot, ".php"))  {
+    setup_php(fd);
   } else {
     setup_c(fd);
   }
