@@ -148,6 +148,8 @@ int main(int argc, char **argv)
                     forcelang = LANG_C;
                 else if (!string_compare(optarg, "perl"))
                     forcelang = LANG_PERL;
+                else if (!string_compare(optarg, "php"))
+                    forcelang = LANG_PHP;
                 else
                     fprintf(stderr, "Language %s unknown, using filename extensions instead\n", optarg);
                 break;
