@@ -36,6 +36,8 @@ enum _type_t
     StaticGlobalBuffer,
     Reference,
     PythonBacktick,
+    PhpBacktick,
+    PerlBacktick,
     None
 };
 
@@ -82,6 +84,8 @@ extern int warning_level;
 extern void         log_staticbuffer(type_t type, int, Severity_t);
 extern void         log_toctou(toctou_t **, int, int, int);
 extern void	    log_pythonbacktick(int, Severity_t);
+extern void         log_perlbacktick(int, Severity_t);
+extern void         log_phpbacktick(int, Severity_t);
 extern void         log_vulnerability(type_t, Severity_t);
 extern void         record_input(void);
 extern void         generate_report(void);
