@@ -24,7 +24,7 @@
 
 #ifdef HAVE_EXPAT_H
 #  include "expat.h"
-#  else 
+#  else HAVE_XMLPARSE_H
 #    ifdef HAVE_XMLPARSE_H
 #      include "xmlparse.h"
 #    endif
@@ -41,6 +41,9 @@ typedef struct udata_t  {
     char databuf[2048];
     char *bufptr;
     Vuln_t *vuln;
+    int failed;
+    int failedat;
+    XML_Parser parser;
     Hash myhash;
     Hash parenthash;
 
@@ -49,8 +52,11 @@ typedef struct udata_t  {
 
 
 
-void InitUdata(udata_t *udata)  {
+void InitUdata(udata_t *udata, XML_Parser parser)  {
 
+    udata->failed = 0;
+    udata->failedat = 0;
+    udata->parser = parser;
     udata->depth = -1;
     udata->bufptr = NULL;
     udata->vuln = NULL;
@@ -83,6 +89,7 @@ void InitVuln(Vuln_t *vuln)  {
     vuln->RaceUse = 0;
     vuln->Input = 0;
     vuln->Info = NULL;
+    vuln->InputProblem = NULL;
 }
 
 
@@ -126,15 +133,28 @@ void InitInfo(Info_t *info)  {
 }
 
 
+static int SetFailed(udata_t *mydata)
+{
+  mydata->failed = 1;
+  mydata->failedat = XML_GetCurrentLineNumber(mydata->parser);
+  return mydata->failedat;
+}
+
+
+    
 void StartElement(void *udata, const char *name, const char **atts)  {
 
     udata_t *mydata = (udata_t *) udata;
     char *langname = NULL;
     char **tmp;
-    char *curname;
+    char *curname = NULL;
     char *curval;
     int count = 0;
     int i;
+    if (mydata->failed)  
+    {
+      return;
+    }
     mydata->fname[mydata->depth+1] = malloc(strlen(name)+1);
     strncpy(mydata->fname[mydata->depth+1], name, strlen(name));
     mydata->fname[mydata->depth+1][strlen(name)] = '\0';  
@@ -172,51 +192,51 @@ void StartElement(void *udata, const char *name, const char **atts)  {
         }
     } else if(!strcmp(name, "Vulnerability"))  {
         if(mydata->vuln != NULL)  {
-            fprintf(stderr, "Vulnerability open tag found, but already in one!?\n");
+            fprintf(stderr, "Vulnerability open tag found, but already in one at line %d\n", SetFailed(mydata));
             return;
         }
         mydata->vuln = malloc(sizeof(Vuln_t));
         InitVuln(mydata->vuln);
     } else if(!strcmp(name, "FSProblem"))  {
         if(mydata->vuln == NULL)  {
-            fprintf(stderr,"In FSProblem, but no Vuln struct\n"); 
+            fprintf(stderr,"In FSProblem, but no Vuln struct at line %d\n", SetFailed(mydata)); 
             return;
         } 
         if(mydata->vuln->FSProblem != NULL)  {
-            fprintf(stderr, "FSProblem open tag found but already in one!?\n");
+            fprintf(stderr, "FSProblem open tag found but already in one at line %d\n", SetFailed(mydata));
             return;
         }              
         mydata->vuln->FSProblem = malloc(sizeof(FSProblem_t));
         InitFSProblem(mydata->vuln->FSProblem);
     } else if(!strcmp(name, "BOProblem"))  {
         if(mydata->vuln == NULL)  {
-            fprintf(stderr, "In BOProblem, but no Vuln struct\n");
+            fprintf(stderr, "In BOProblem, but no Vuln struct at line %d\n", SetFailed(mydata));
             return;
         }
         if(mydata->vuln->BOProblem != NULL)  {
-            fprintf(stderr,"BOProblem open tag found, but already in one!?\n");
+            fprintf(stderr,"BOProblem open tag found, but already in one at line %d\n", SetFailed(mydata));
             return;
         }
         mydata->vuln->BOProblem = malloc(sizeof(BOProblem_t));
         InitBOProblem(mydata->vuln->BOProblem);
     } else if(!strcmp(name, "InputProblem"))  {
         if(mydata->vuln == NULL)  {
-            fprintf(stderr, "In InputProblem, but no Vuln struct\n");
+            fprintf(stderr, "In InputProblem, but no Vuln struct at line %d\n", SetFailed(mydata));
             return;
         }
         if(mydata->vuln->InputProblem != NULL)  {
-            fprintf(stderr,"InputProblem open tag found, but already in one!?\n");
+            fprintf(stderr,"InputProblem open tag found, but already in one at %d\n", SetFailed(mydata));
             return;
         }
         mydata->vuln->InputProblem = malloc(sizeof(InputProblem_t));
         InitInputProblem(mydata->vuln->InputProblem);
     } else if (!strcmp(name, "Info"))  {
         if(mydata->vuln == NULL)  {
-            fprintf(stderr,"In Info, but no Vuln struct\n");
+            fprintf(stderr,"In Info, but no Vuln struct at line %d\n", SetFailed(mydata));
             return;
         }
         if(mydata->vuln->Info != NULL)  {
-            fprintf(stderr, "Found Info open tag, but already in one!?\n");
+            fprintf(stderr, "Found Info open tag, but already in one at line %d\n", SetFailed(mydata));
             return;
         }
         mydata->vuln->Info = malloc(sizeof(Info_t));
@@ -251,6 +271,8 @@ void EndElement(void *udata, const char *name)  {
 
     udata_t *mydata = (udata_t *)udata;
     
+    if (mydata->failed)
+      return;
     free(mydata->fname[mydata->depth]);
     mydata->depth--;
     if(!strcmp(name, "Vulnerability"))  {
@@ -313,6 +335,10 @@ void EndElement(void *udata, const char *name)  {
 void CharData(void *udata, const XML_Char *str, int len)  {
 
     udata_t *mydata = (udata_t *)udata;
+    if (mydata->failed)
+    {
+      return;
+    }
     if(mydata->bufptr + len > mydata->databuf+sizeof(mydata->databuf))  {
         fprintf(stderr, "Possible data overflow, ignoring it.\n");
         return;
@@ -337,7 +363,7 @@ Hash ParseVulnDb(char *buf, Hash *usehash)  {
         fprintf(stderr, "Failed to create XML Parser\n");
         return NULL;
     }
-    InitUdata(&foo);
+    InitUdata(&foo, parser);
     XML_SetUserData(parser, &foo);
     XML_SetElementHandler(parser, StartElement, EndElement);
     XML_SetCharacterDataHandler(parser, CharData);
@@ -357,6 +383,7 @@ Hash ParseVulnDb(char *buf, Hash *usehash)  {
               XML_GetCurrentLineNumber(parser));
         return NULL;
     }
-
+    if (foo.failed)
+      return NULL;
     return *usehash;
 }
